@@ -10,7 +10,10 @@ import numpy as np
 import scipp as sc
 
 from ._common import to_child_select
+from .nxcylindrical_geometry import NXcylindrical_geometry
 from .nxobject import Field, NexusStructureError, NXobject, ScippIndex, asarray
+from .nxoff_geometry import NXoff_geometry
+from .nxtransformations import NXtransformations
 from .typing import H5Group
 
 
@@ -31,7 +34,8 @@ class NXdataStrategy:
     def signal(group):
         """Name of the signal field."""
         if (name := group.attrs.get('signal')) is not None:
-            return name
+            if name in group:
+                return name
         # Legacy NXdata defines signal not as group attribute, but attr on dataset
         for name in group.keys():
             # What is the meaning of the attribute value? It is undocumented, we simply
@@ -99,6 +103,13 @@ class NXdata(NXobject):
         # corresponding coords. Special case of '.' entries means "no coord".
         if (axes := self._strategy.axes(self)) is not None:
             return [f'dim_{i}' if a == '.' else a for i, a in enumerate(axes)]
+        axes = []
+        # Names of axes that have an "axis" attribute serve as dim labels in legacy case
+        for name, field in self._group.items():
+            if (axis := field.attrs.get('axis')) is not None:
+                axes.append((axis, name))
+        if axes:
+            return [x[1] for x in sorted(axes)]
         return None
 
     @property
@@ -126,6 +137,9 @@ class NXdata(NXobject):
         if self._signal_override is not None:
             return self._signal_override
         if self._signal_name is not None:
+            if self._signal_name not in self:
+                raise NexusStructureError(
+                    f"Signal field '{self._signal_name}' not found in group.")
             return self[self._signal_name]
         return None
 
@@ -171,6 +185,8 @@ class NXdata(NXobject):
         # since legacy files do not set this attribute.
         if (indices := self.attrs.get(f'{name}_indices')) is not None:
             return list(np.array(self.dims)[np.array(indices).flatten()])
+        if (axis := self._get_child(name).attrs.get('axis')) is not None:
+            return (self._get_group_dims()[axis - 1], )
         if name in [self._signal_name, self._errors_name]:
             return self._get_group_dims()  # if None, field determines dims itself
         if name in list(self.attrs.get('auxiliary_signals', [])):
@@ -178,6 +194,11 @@ class NXdata(NXobject):
         if name in self._get_axes():
             # If there are named axes then items of same name are "dimension
             # coordinates", i.e., have a dim matching their name.
+            # However, if the item is not 1-D we need more labels. Try to use labels of
+            # signal if dimensionality matches.
+            if self._signal_name in self and self._get_child(name).ndim == len(
+                    self.shape):
+                return self[self._signal_name].dims
             return [name]
         return self._try_guess_dims(name)
 
@@ -204,13 +225,18 @@ class NXdata(NXobject):
         return False
 
     def _getitem(self, select: ScippIndex) -> sc.DataArray:
+        from .nexus_classes import NXgeometry
         signal = self._signal
         if signal is None:
-            raise NexusStructureError("No signal field found, cannot load group")
+            raise NexusStructureError("No signal field found, cannot load group.")
         signal = signal[select]
         if self._errors_name is not None:
             stddevs = self[self._errors_name][select]
-            signal.variances = sc.pow(stddevs, sc.scalar(2)).values
+            # According to the standard, errors must have the same shape as the data.
+            # This is not the case in all files we observed, is there any harm in
+            # attempting a broadcast?
+            signal.variances = np.broadcast_to(sc.pow(stddevs, sc.scalar(2)).values,
+                                               shape=signal.shape)
 
         da = sc.DataArray(data=signal) if isinstance(signal, sc.Variable) else signal
 
@@ -220,6 +246,17 @@ class NXdata(NXobject):
         for name in self:
             if (errors := self._strategy.coord_errors(self, name)) is not None:
                 skip += [errors]
+        for name in self:
+            if name in skip:
+                continue
+            # It is not entirely clear whether skipping NXtransformations is the right
+            # solution. In principle NXobject will load them via the 'depends_on'
+            # mechanism, so for valid files this should be sufficient.
+            allowed = (Field, NXtransformations, NXcylindrical_geometry, NXoff_geometry,
+                       NXgeometry)
+            if not isinstance(self._get_child(name), allowed):
+                raise NexusStructureError(
+                    "Invalid NXdata: may not contain nested groups")
 
         for name, field in self[Field].items():
             if name in skip:
@@ -232,11 +269,16 @@ class NXdata(NXobject):
             if (error_name := self._strategy.coord_errors(self, name)) is not None:
                 stddevs = asarray(self[error_name][sel])
                 coord.variances = sc.pow(stddevs, sc.scalar(2)).values
-            if self._coord_to_attr(da, name, field):
-                # Like scipp, slicing turns coord into attr if slicing removes the
-                # dim corresponding to the coord.
-                da.attrs[name] = coord
-            else:
-                da.coords[name] = coord
+            try:
+                if self._coord_to_attr(da, name, field):
+                    # Like scipp, slicing turns coord into attr if slicing removes the
+                    # dim corresponding to the coord.
+                    da.attrs[name] = coord
+                else:
+                    da.coords[name] = coord
+            except sc.DimensionError as e:
+                raise NexusStructureError(
+                    f"Field in NXdata incompatible with dims or shape of signal: {e}"
+                ) from e
 
         return da
