@@ -4,7 +4,7 @@
 from __future__ import annotations
 
 from copy import copy
-from typing import List, Optional, Union
+from typing import Any, Dict, List, Optional, Union
 
 import scipp as sc
 
@@ -72,6 +72,10 @@ class _EventField:
         self._grouping_key = grouping_key
         self._grouping = grouping
 
+    @property
+    def name(self) -> str:
+        return self._nxevent_data.name
+
     @property
     def attrs(self):
         return self._nxevent_data.attrs
@@ -91,11 +95,13 @@ class _EventField:
         return self._grouping.shape
 
     @property
-    def unit(self):
-        self._nxevent_data.unit
+    def unit(self) -> None:
+        return self._nxevent_data.unit
 
     def __getitem__(self, select: ScippIndex) -> sc.DataArray:
-        event_data: sc.DataArray = self._nxevent_data[self._event_select]
+        event_data = self._nxevent_data[self._event_select]
+        if isinstance(event_data, sc.DataGroup):
+            raise NexusStructureError("Invalid NXevent_data in NXdetector.")
         if self._grouping is None:
             if select not in (Ellipsis, tuple(), slice(None)):
                 raise NexusStructureError(
@@ -164,7 +170,14 @@ class NXdetector(NXobject):
         return self._signal.unit
 
     @property
-    def _event_grouping(self) -> Union[None, Field]:
+    def detector_number(self) -> Optional[Field]:
+        for key in self._detector_number_fields:
+            if key in self:
+                return key
+        return None
+
+    @property
+    def _event_grouping(self) -> Union[None, Dict[str, Any]]:
         for key in self._detector_number_fields:
             if key in self:
                 return {'grouping_key': key, 'grouping': self[key]}
@@ -175,16 +188,21 @@ class NXdetector(NXobject):
         return self._nxdata()._signal
 
     def _nxdata(self, use_event_signal=True) -> NXdata:
-        if use_event_signal and self.events is not None:
-            signal = _EventField(self.events, self._event_select,
-                                 **self._event_grouping)
+        events = self.events
+        if use_event_signal and events is not None:
+            signal = _EventField(events, self._event_select, **self._event_grouping)
         else:
             signal = None
-        return NXdata(
-            self._group,
-            strategy=NXdetectorStrategy,
-            signal_override=signal,
-            skip=self._nxevent_data_fields if self.events is not None else None)
+        skip = None
+        if events is not None:
+            if events.name == self.name:
+                skip = self._nxevent_data_fields
+            else:
+                skip = [events.name.split('/')[-1]]  # name of the subgroup
+        return NXdata(self._group,
+                      strategy=NXdetectorStrategy,
+                      signal_override=signal,
+                      skip=skip)
 
     @property
     def events(self) -> Union[None, NXevent_data]:
