@@ -137,16 +137,34 @@ def _as_datetime(obj: Any):
     return None
 
 
+def _dtype_from_dataset(dataset: H5Dataset) -> sc.DType:
+    dtype = dataset.dtype
+    if str(dtype).startswith('str') or h5py.check_string_dtype(dtype):
+        dtype = sc.DType.string
+    else:
+        dtype = sc.DType(_ensure_supported_int_type(str(dtype)))
+    return dtype
+
+
 class Field:
     """NeXus field.
 
     In HDF5 fields are represented as dataset.
     """
 
-    def __init__(self, dataset: H5Dataset, *, ancestor, dims=None, is_time=None):
+    def __init__(self,
+                 dataset: H5Dataset,
+                 *,
+                 ancestor,
+                 dims=None,
+                 dtype: Optional[sc.DType] = None,
+                 is_time=None):
         self._ancestor = ancestor  # Usually the parent, but may be grandparent, etc.
         self._dataset = dataset
+        self._dtype = _dtype_from_dataset(dataset) if dtype is None else dtype
         self._shape = self._dataset.shape
+        if self._dtype == sc.DType.vector3:
+            self._shape = self._shape[:-1]
         self._is_time = is_time
         # NeXus treats [] and [1] interchangeably. In general this is ill-defined, but
         # the best we can do appears to be squeezing unless the file provides names for
@@ -242,13 +260,8 @@ class Field:
         return Attrs(self._dataset.attrs)
 
     @property
-    def dtype(self) -> str:
-        dtype = self._dataset.dtype
-        if str(dtype).startswith('str') or h5py.check_string_dtype(dtype):
-            dtype = sc.DType.string
-        else:
-            dtype = sc.DType(_ensure_supported_int_type(str(dtype)))
-        return dtype
+    def dtype(self) -> sc.DType:
+        return self._dtype
 
     @property
     def name(self) -> str:
@@ -362,45 +375,67 @@ class NXobject:
         if isinstance(name, str):
             item = self._group[name]
             if is_dataset(item):
-                dims = self._get_field_dims(name) if use_field_dims else None
+                try:
+                    dims = self._get_field_dims(name) if use_field_dims else None
+                except Exception as e:
+                    msg = (f"Failed to determine axis names of {item.name}: {e}. "
+                           "Falling back to default dimension labels.")
+                    warnings.warn(msg)
+                    dims = None
+                dtype = self._get_field_dtype(name)
                 return Field(item,
                              dims=dims,
+                             dtype=dtype,
                              ancestor=self,
                              **self.child_params.get(name, {}))
             else:
                 return self._make(item)
+
         try:
             da = self._getitem(name)
-        except Exception as e:
+            self._insert_leaf_properties(da)
+        except NexusStructureError as e:
             # If the child class cannot load this group, we fall back to returning the
             # underlying datasets in a DataGroup.
             if type(self)._getitem == NXobject._getitem:
                 raise
             else:
-                warnings.warn(
-                    f"Failed to load {self.name} as {type(self).__name__}:\n{e}\n"
+                msg = (
+                    f"Failed to load {self.name} as {type(self).__name__}: {e} "
                     "Falling back to loading HDF5 group children as scipp.DataGroup.")
+                warnings.warn(msg)
             da = NXobject._getitem(self, name)
+        return da
+
+    def _insert_leaf_properties(self, container):
+        from .nexus_classes import NXgeometry
+        from .nxcylindrical_geometry import NXcylindrical_geometry
+        from .nxoff_geometry import NXoff_geometry
+
+        def insert(container, name, obj):
+            if hasattr(container, 'coords'):
+                container.coords[name] = obj if isinstance(
+                    obj, sc.Variable) else sc.scalar(obj)
+            else:
+                container[name] = obj
+
+        detector_number = getattr(self, 'detector_number', None)
+        if detector_number is not None:
+            detector_number = container.coords[detector_number]
+        for key, child in self[[NXcylindrical_geometry, NXoff_geometry]].items():
+            insert(container, key, child.load_as_array(detector_number=detector_number))
+        for key, child in self[NXgeometry].items():
+            insert(container, key, child[()])
         if (t := self.depends_on) is not None:
-
-            def insert(name, obj):
-                if hasattr(da, 'coords'):
-                    da.coords[name] = obj if isinstance(obj,
-                                                        sc.Variable) else sc.scalar(obj)
-                else:
-                    da[name] = obj
-
-            insert('depends_on', t)
+            insert(container, 'depends_on', t)
             # If loading the transformation failed, 'depends_on' returns a string, the
             # path to the transformation. If this is a nested group, we load it here.
             # Note that this info is currently incomplete, since attributes are not
             # loaded.
             if isinstance(t, str):
                 from .nexus_classes import NXtransformations
-                for name, group in self[NXtransformations].items():
-                    insert(name, group[()])
-
-        return da
+                for key, group in self[NXtransformations].items():
+                    insert(container, key, group[()])
 
     def _get_children_by_nx_class(
             self, select: Union[type,
@@ -422,7 +457,7 @@ class NXobject:
         ...
 
     @overload
-    def __getitem__(self, name: Union[type, Tuple[type]]) -> Dict[str, 'NXobject']:
+    def __getitem__(self, name: Union[type, List[type]]) -> Dict[str, 'NXobject']:
         ...
 
     def __getitem__(self, name):
@@ -469,6 +504,10 @@ class NXobject:
         """Subclasses should reimplement this to provide dimension labels for fields."""
         return None
 
+    def _get_field_dtype(self, name: str) -> Union[None, sc.DType]:
+        """Subclasses should reimplement this to override the dtype for fields."""
+        return None
+
     def __contains__(self, name: str) -> bool:
         return name in self._group
 
