@@ -14,7 +14,7 @@ from scippnexus import (
 
 
 @pytest.fixture()
-def nxroot(request):
+def nxroot():
     """Yield NXroot containing a single NXentry named 'entry'"""
     with h5py.File('dummy.nxs', mode='w', driver="core", backing_store=False) as f:
         root = NXroot(f)
@@ -77,10 +77,8 @@ def test_chain_with_single_values_and_different_unit(nxroot):
     value2.attrs['transformation_type'] = 'translation'
     value2.attrs['vector'] = vector.value
 
-    expected = sc.spatial.affine_transform(value=np.identity(4), unit=t.unit)
-    expected = expected * sc.spatial.translations(
-        dims=t.dims, values=2 * t.values, unit=t.unit)
-    expected = expected * sc.spatial.translation(value=[0.001, 0.002, 0.003], unit='m')
+    expected = (sc.spatial.translations(dims=t.dims, values=2 * t.values, unit=t.unit) *
+                sc.spatial.translation(value=[0.001, 0.002, 0.003], unit='m'))
     assert sc.identical(detector[...].coords['depends_on'], expected)
 
 
@@ -141,8 +139,7 @@ def test_chain_with_multiple_values(nxroot):
     value2.attrs['transformation_type'] = 'translation'
     value2.attrs['vector'] = vector.value
 
-    expected = sc.spatial.affine_transform(value=np.identity(4), unit=t.unit)
-    expected = t * (t * (offset * expected))
+    expected = t * (t * offset)
     assert sc.identical(detector[...].coords['depends_on'].value, expected)
 
 
@@ -175,8 +172,7 @@ def test_chain_with_multiple_values_and_different_time_unit(nxroot):
     value2.attrs['transformation_type'] = 'translation'
     value2.attrs['vector'] = vector.value
 
-    expected = sc.spatial.affine_transform(value=np.identity(4), unit=t.unit)
-    expected = t * (t * (offset * expected))
+    expected = t * (t * offset)
     assert sc.identical(detector[...].coords['depends_on'].value, expected)
 
 
@@ -211,3 +207,100 @@ def test_broken_time_dependent_transformation_returns_path_and_transformations(n
     # attributes. DataGroup does currently not support attributes, so this information
     # is mostly useless until that is addressed.
     assert 't1' in t
+
+
+def write_translation(group, name: str, value: sc.Variable, offset: sc.Variable,
+                      vector: sc.Variable) -> None:
+    dset = group.create_field(name, value)
+    dset.attrs['transformation_type'] = 'translation'
+    dset.attrs['offset'] = offset.values
+    dset.attrs['offset_units'] = str(offset.unit)
+    dset.attrs['vector'] = vector.value
+
+
+def test_nxtransformations_group_single_item(nxroot):
+    value = sc.scalar(2.4, unit='mm')
+    offset = sc.spatial.translation(value=[6, 2, 6], unit='mm')
+    vector = sc.vector(value=[0, 1, 1])
+    t = value.to(unit='m') * vector
+    expected = (sc.spatial.translations(dims=t.dims, values=t.values, unit=t.unit) *
+                sc.spatial.translation(value=[0.006, 0.002, 0.006], unit='m'))
+
+    transformations = nxroot.create_class('transformations', NXtransformations)
+    write_translation(transformations, 't1', value, offset, vector)
+
+    loaded = nxroot['transformations'][()]
+    assert set(loaded.keys()) == {'t1'}
+    assert sc.identical(loaded['t1'], expected)
+
+
+def test_nxtransformations_group_two_independent_items(nxroot):
+    transformations = nxroot.create_class('transformations', NXtransformations)
+
+    value = sc.scalar(2.4, unit='mm')
+    offset = sc.spatial.translation(value=[6, 2, 6], unit='mm')
+    vector = sc.vector(value=[0, 1, 1])
+    t = value.to(unit='m') * vector
+    write_translation(transformations, 't1', value, offset, vector)
+    expected1 = (sc.spatial.translations(dims=t.dims, values=t.values, unit=t.unit) *
+                 sc.spatial.translation(value=[0.006, 0.002, 0.006], unit='m'))
+
+    value = value * 0.1
+    t = value.to(unit='m') * vector
+    write_translation(transformations, 't2', value, offset, vector)
+    expected2 = (sc.spatial.translations(dims=t.dims, values=t.values, unit=t.unit) *
+                 sc.spatial.translation(value=[0.006, 0.002, 0.006], unit='m'))
+
+    loaded = nxroot['transformations'][()]
+    assert set(loaded.keys()) == {'t1', 't2'}
+    assert sc.identical(loaded['t1'], expected1)
+    assert sc.identical(loaded['t2'], expected2)
+
+
+def test_nxtransformations_group_single_chain(nxroot):
+    transformations = nxroot.create_class('transformations', NXtransformations)
+
+    value = sc.scalar(2.4, unit='mm')
+    offset = sc.spatial.translation(value=[6, 2, 6], unit='mm')
+    vector = sc.vector(value=[0, 1, 1])
+    t = value.to(unit='m') * vector
+    write_translation(transformations, 't1', value, offset, vector)
+    expected1 = (sc.spatial.translations(dims=t.dims, values=t.values, unit=t.unit) *
+                 sc.spatial.translation(value=[0.006, 0.002, 0.006], unit='m'))
+
+    value = value * 0.1
+    t = value.to(unit='m') * vector
+    write_translation(transformations, 't2', value, offset, vector)
+    transformations['t2'].attrs['depends_on'] = 't1'
+    expected2 = (expected1 *
+                 sc.spatial.translations(dims=t.dims, values=t.values, unit=t.unit) *
+                 sc.spatial.translation(value=[0.006, 0.002, 0.006], unit='m'))
+
+    loaded = nxroot['transformations'][()]
+    assert set(loaded.keys()) == {'t1', 't2'}
+    assert sc.identical(loaded['t1'], expected1)
+    assert sc.allclose(loaded['t2'], expected2)
+
+
+def test_slice_transformations(nxroot):
+    transformations = nxroot.create_class('transformations', NXtransformations)
+    log = sc.DataArray(
+        sc.array(dims=['time'], values=[1.1, 2.2, 3.3], unit='m'),
+        coords={'time': sc.array(dims=['time'], values=[11, 22, 33], unit='s')})
+    log.coords['time'] = sc.epoch(unit='ns') + log.coords['time'].to(unit='ns')
+    offset = sc.spatial.translation(value=[1, 2, 3], unit='m')
+    vector = sc.vector(value=[0, 0, 1])
+    t = log * vector
+    t.data = sc.spatial.translations(dims=t.dims, values=t.values, unit=t.unit)
+    value1 = transformations.create_class('t1', NXlog)
+    value1['time'] = log.coords['time'] - sc.epoch(unit='ns')
+    value1['value'] = log.data
+    value1.attrs['transformation_type'] = 'translation'
+    value1.attrs['offset'] = offset.values
+    value1.attrs['offset_units'] = str(offset.unit)
+    value1.attrs['vector'] = vector.value
+
+    expected = t * offset
+
+    assert sc.identical(nxroot['transformations']['time', 1:3]['t1'], expected['time',
+                                                                               1:3])
